@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -29,6 +29,8 @@
 
 #include "gstchannelmix.h"
 
+#define INT_MATRIX_FACTOR_EXPONENT 10
+
 /*
  * Channel matrix functions.
  */
@@ -48,6 +50,13 @@ gst_channel_mix_unset_matrix (AudioConvertCtx * this)
   g_free (this->matrix);
 
   this->matrix = NULL;
+
+  for (i = 0; i < this->in.channels; i++)
+    g_free (this->matrix_int[i]);
+  g_free (this->matrix_int);
+
+  this->matrix_int = NULL;
+
   g_free (this->tmp);
   this->tmp = NULL;
 }
@@ -590,6 +599,26 @@ gst_channel_mix_fill_matrix (AudioConvertCtx * this)
   }
 }
 
+/* only call this after this->matrix is fully set up and normalized */
+static void
+gst_channel_mix_setup_matrix_int (AudioConvertCtx * this)
+{
+  gint i, j;
+  gfloat tmp;
+  gfloat factor = (1 << INT_MATRIX_FACTOR_EXPONENT);
+
+  this->matrix_int = g_new0 (gint *, this->in.channels);
+
+  for (i = 0; i < this->in.channels; i++) {
+    this->matrix_int[i] = g_new (gint, this->out.channels);
+
+    for (j = 0; j < this->out.channels; j++) {
+      tmp = this->matrix[i][j] * factor;
+      this->matrix_int[i][j] = (gint)tmp;
+    }
+  }
+}
+
 /* only call after this->out and this->in are filled in */
 void
 gst_channel_mix_setup_matrix (AudioConvertCtx * this)
@@ -618,6 +647,8 @@ gst_channel_mix_setup_matrix (AudioConvertCtx * this)
   /* setup the matrix' internal values */
   gst_channel_mix_fill_matrix (this);
 
+  gst_channel_mix_setup_matrix_int(this);
+
 #ifndef GST_DISABLE_GST_DEBUG
   /* debug */
   {
@@ -654,6 +685,10 @@ gst_channel_mix_passthrough (AudioConvertCtx * this)
   if (this->in.channels != this->out.channels)
     return FALSE;
 
+  /* passthrough for 1->1 channels (MONO and NONE position are the same here) */
+  if (this->in.channels == 1 && this->out.channels == 1)
+    return TRUE;
+
   /* passthrough if both channel masks are the same */
   in_mask = out_mask = 0;
   for (i = 0; i < this->in.channels; i++) {
@@ -690,9 +725,12 @@ gst_channel_mix_mix_int (AudioConvertCtx * this,
       /* convert */
       res = 0;
       for (in = 0; in < inchannels; in++) {
-        res += in_data[n * inchannels + in] * this->matrix[in][out];
+        res += in_data[n * inchannels + in] * (gint64)this->matrix_int[in][out];
       }
 
+      /* remove factor from int matrix */
+      res = res >> INT_MATRIX_FACTOR_EXPONENT;
+
       /* clip (shouldn't we use doubles instead as intermediate format?) */
       if (res < G_MININT32)
         res = G_MININT32;
